﻿/*

The MIT License

Copyright (c) 2006 Elvis Mehmedović

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

http://emehmedovic.com/

*/

import xm.corpus.resizableComponent;
import flash.display.BitmapData;
import flash.geom.Matrix;

class xm.components.BitmapTileComponent extends resizableComponent {

	[IconFile("TileComponent.png")]

	private var __bitmapSymbol:String;
	private var __bitmap:BitmapData;
	private var __url:String;

	private var _initializing:Boolean = true;

	private var _imageContainer:MovieClip;
	private var _depthCounter:Number = 0;

	var matrix:Matrix;
	var repeat:Boolean = true;
	var resetMatrix:Boolean = true;

	[Inspectable()]
	var smoothing:Boolean = false;

	var tileContainer:MovieClip;
	var loader:MovieClipLoader;

	[Inspectable()]
	function get bitmapSymbol():String {
		return __bitmapSymbol;
	}

	function set bitmapSymbol(p_val:String) {

		__bitmapSymbol = p_val;

		if (!tileContainer) {
			return;
		}

		if (resetMatrix) {
			matrix = new Matrix(1, 0, 0, 1, 0, 0);
		}
		bitmap = BitmapData.loadBitmap(bitmapSymbol);
	}

	function flush(Void):Void {
		__bitmap = null;
		tileContainer.clear();
	}

	[Inspectable()]
	function get url():String {
		return __url;
	}

	function set url(p_val:String) {

		__url = p_val;

		if (!tileContainer || p_val == "" || p_val == undefined) {
			return;
		}

		if (loader) {
			loader.removeListener(this);
		}

		loader = new MovieClipLoader();
		loader.addListener(this);

		_imageContainer.removeMovieClip();
		_imageContainer = _imageContainer.createEmptyMovieClip(String(++_depthCounter), _depthCounter);

		loader.loadClip(url, _imageContainer);

	}

	private function onLoadInit(p_mc:MovieClip):Void {

		var lbmp:BitmapData = new BitmapData(_imageContainer._width, _imageContainer._height);

		lbmp.draw(_imageContainer);
		_imageContainer.removeMovieClip();
		
		if (resetMatrix) {
			matrix = new Matrix(1, 0, 0, 1, 0, 0);
		}
		bitmap = lbmp;
	}

	function get bitmap():BitmapData {
		return __bitmap;
	}

	function set bitmap(p_val:BitmapData) {

		__bitmap = p_val;

		if (!tileContainer) {
			return;
		}

		tileContainer.clear();

		if (bitmap.width) {
			tileContainer.beginBitmapFill(bitmap, matrix, repeat, smoothing);
			tileContainer.lineTo(width, 0);
			tileContainer.lineTo(width, height);
			tileContainer.lineTo(0, height);
			tileContainer.lineTo(0, 0);
			tileContainer.endFill();
		}
		else {
			flush();
		}

	}
	
	function refresh(Void):Void {
/*		var bmp:BitmapData = bitmap;
		bitmap = null;*/
		bitmap = bitmap;
	}

	function setSize(p_width:Number, p_height:Number):Void {
		super.setSize(p_width, p_height);
		refresh();
	}

	function DABitmapTileComponent() {
	}

	function init(Void):Void {

		super.init();
		nextFrame();

		tileContainer = createEmptyMovieClip(String(++_depthCounter), _depthCounter);

		bitmap = bitmap;
		bitmapSymbol = bitmapSymbol;
		url = url;

		_initializing = false;

	}

}